/*
 * ByteDance Volcengine EMR, Copyright 2022.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.fs.tosfs;

import org.apache.hadoop.fs.FileChecksum;
import org.apache.hadoop.fs.tosfs.common.Bytes;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

public class TosChecksum extends FileChecksum {
  private String algorithm;
  private byte[] checksum;

  public TosChecksum(String algorithm, byte[] checksum) {
    this.algorithm = algorithm;
    this.checksum = checksum;
  }

  @Override
  public String getAlgorithmName() {
    return algorithm;
  }

  @Override
  public int getLength() {
    return checksum.length;
  }

  @Override
  public byte[] getBytes() {
    return checksum;
  }

  @Override
  public void write(DataOutput out) throws IOException {
    byte[] algorithmBytes = Bytes.toBytes(algorithm);
    out.write(algorithmBytes.length);
    out.write(algorithmBytes);
    out.write(checksum.length);
    out.write(checksum);
  }

  @Override
  public void readFields(DataInput in) throws IOException {
    byte[] algorithmBytes = new byte[in.readInt()];
    in.readFully(algorithmBytes);
    algorithm = Bytes.toString(algorithmBytes);
    checksum = new byte[in.readInt()];
    in.readFully(checksum);
  }
}
