/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.yarn.server.webapp;

import org.apache.hadoop.yarn.conf.YarnConfiguration;
import org.apache.hadoop.yarn.logaggregation.filecontroller.LogAggregationFileControllerFactory;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.fail;
import static org.mockito.Mockito.mock;

public class TestLogServlet {
  /**
   * Test that {@link LogServlet}'s constructor does not throw exception,
   * if the log aggregation properties are bad.
   */
  @Test
  public void testLogServletNoException() {
    YarnConfiguration conf = new YarnConfiguration();
    conf.set(YarnConfiguration.LOG_AGGREGATION_FILE_FORMATS, "22");

    // first test the factory's constructor throws exception
    try {
      LogAggregationFileControllerFactory factory =
          new LogAggregationFileControllerFactory(conf);
      fail("LogAggregationFileControllerFactory should have thrown exception");
    } catch (IllegalArgumentException expected) {
    }

    // LogServlet should not throw exception
    AppInfoProvider aip = mock(AppInfoProvider.class);
    LogServlet ls = new LogServlet(conf, aip);
    assertThat(ls).isNotNull();
  }
}
