/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.hadoop.yarn.server.nodemanager.containermanager.linux.runtime.docker;

import org.apache.hadoop.util.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * Tests the docker images command and its command
 * line arguments.
 */
public class TestDockerImagesCommand {
  private DockerImagesCommand dockerImagesCommand;

  private static final String IMAGE_NAME = "foo";

  @BeforeEach
  public void setup() {
    dockerImagesCommand = new DockerImagesCommand();
  }

  @Test
  public void testGetCommandOption() {
    assertEquals("images", dockerImagesCommand.getCommandOption());
  }

  @Test
  public void testAllImages() {
    assertEquals("images", StringUtils.join(",",
        dockerImagesCommand.getDockerCommandWithArguments()
            .get("docker-command")));
    assertEquals(1, dockerImagesCommand.getDockerCommandWithArguments().size());
  }

  @Test
  public void testSingleImage() {
    dockerImagesCommand = dockerImagesCommand.getSingleImageStatus(IMAGE_NAME);
    assertEquals("images", StringUtils.join(",",
        dockerImagesCommand.getDockerCommandWithArguments()
            .get("docker-command")));
    assertEquals("foo",
        StringUtils.join(",", dockerImagesCommand.getDockerCommandWithArguments().get("image")),
        "image name");
    assertEquals(2, dockerImagesCommand.getDockerCommandWithArguments().size());
  }
}
